@@ -1,3 +1,7 @@
+from collections import namedtuple
+from typing import Any, Dict, List, Optional, Tuple
+
+from elasticsearch import Elasticsearch
 from elasticsearch import exceptions as es_exceptions
 from es import exceptions
 from six import string_types
@@ -7,6 +11,14 @@ from six.moves.urllib import parse
 from .const import DEFAULT_FETCH_SIZE, DEFAULT_SCHEMA, DEFAULT_SQL_PATH
 
 
+CursorDescriptionRow = namedtuple(
+    "CursorDescriptionRow",
+    ["name", "type", "display_size", "internal_size", "precision", "scale", "null_ok"],
+)
+
+CursorDescriptionType = List[CursorDescriptionRow]
+
+
 class Type(object):
     STRING = 1
     NUMBER = 2
@@ -17,25 +29,80 @@ class Type(object):
 def check_closed(f):
     """Decorator that checks if connection/cursor is closed."""
 
-    def g(self, *args, **kwargs):
+    def wrap(self, *args, **kwargs):
         if self.closed:
             raise exceptions.Error(
                 "{klass} already closed".format(klass=self.__class__.__name__)
             )
         return f(self, *args, **kwargs)
 
-    return g
+    return wrap
 
 
 def check_result(f):
     """Decorator that checks if the cursor has results from `execute`."""
 
-    def g(self, *args, **kwargs):
+    def wrap(self, *args, **kwargs):
         if self._results is None:
             raise exceptions.Error("Called before `execute`")
         return f(self, *args, **kwargs)
 
-    return g
+    return wrap
+
+
+def get_type(data_type) -> int:
+    type_map = {
+        "text": Type.STRING,
+        "keyword": Type.STRING,
+        "integer": Type.NUMBER,
+        "half_float": Type.NUMBER,
+        "scaled_float": Type.NUMBER,
+        "geo_point": Type.STRING,
+        # TODO get a solution for nested type
+        "nested": Type.STRING,
+        "object": Type.STRING,
+        "date": Type.DATETIME,
+        "datetime": Type.DATETIME,
+        "short": Type.NUMBER,
+        "long": Type.NUMBER,
+        "float": Type.NUMBER,
+        "double": Type.NUMBER,
+        "bytes": Type.NUMBER,
+        "boolean": Type.BOOLEAN,
+        "ip": Type.STRING,
+        "interval_minute_to_second": Type.STRING,
+        "interval_hour_to_second": Type.STRING,
+        "interval_hour_to_minute": Type.STRING,
+        "interval_day_to_second": Type.STRING,
+        "interval_day_to_minute": Type.STRING,
+        "interval_day_to_hour": Type.STRING,
+        "interval_year_to_month": Type.STRING,
+        "interval_second": Type.STRING,
+        "interval_minute": Type.STRING,
+        "interval_day": Type.STRING,
+        "interval_month": Type.STRING,
+        "interval_year": Type.STRING,
+    }
+    return type_map[data_type.lower()]
+
+
+def get_description_from_columns(
+    columns: List[Dict[str, str]]
+) -> CursorDescriptionType:
+    return [
+        (
+            CursorDescriptionRow(
+                column.get("name") if not column.get("alias") else column.get("alias"),
+                get_type(column.get("type")),
+                None,  # [display_size]
+                None,  # [internal_size]
+                None,  # [precision]
+                None,  # [scale]
+                True,  # [null_ok]
+            )
+        )
+        for column in columns
+    ]
 
 
 class BaseConnection(object):
@@ -44,21 +111,21 @@ class BaseConnection(object):
 
     def __init__(
         self,
-        host="localhost",
-        port=9200,
-        path="",
-        scheme="http",
-        user=None,
-        password=None,
-        context=None,
-        **kwargs,
+        host: str = "localhost",
+        port: int = 9200,
+        path: str = "",
+        scheme: str = "http",
+        user: Optional[str] = None,
+        password: Optional[str] = None,
+        context: Optional[Dict[Any, Any]] = None,
+        **kwargs: Any,
     ):
         netloc = f"{host}:{port}"
         path = path or "/"
         self.url = parse.urlunparse((scheme, netloc, path, None, None, None))
         self.context = context or {}
         self.closed = False
-        self.cursors = []
+        self.cursors: List[BaseCursor] = []
         self.kwargs = kwargs
         # Subclass needs to initialize Elasticsearch
         self.es = None
@@ -98,11 +165,27 @@ class BaseConnection(object):
         self.close()
 
 
-class BaseCursor(object):
-
+class BaseCursor:
     """Connection cursor."""
 
-    def __init__(self, url, es, **kwargs):
+    custom_sql_to_method: Dict[str, str] = {}
+    """
+    Each child implements custom SQL commands so that we can
+    add extra missing logic or restrictions.
+    Maps custom SQL to class methods, cursor execute calls a dispatcher
+    based on this mapping.
+    """
+
+    def __init__(self, url: str, es: Elasticsearch, **kwargs):
+        """
+        Base cursor constructor initializes common properties
+        that are shared by opendistro and elastic. Child just
+        override the sql_path since they differ on each distribution
+
+        :param url: The connection URL
+        :param es: An initialized Elasticsearch object
+        :param kwargs: connection string query arguments
+        """
         self.url = url
         self.es = es
         self.sql_path = kwargs.get("sql_path") or DEFAULT_SQL_PATH
@@ -114,25 +197,38 @@ class BaseCursor(object):
 
         self.closed = False
 
-        # this is updated only after a query
-        self.description = None
+        # this is updated after a query
+        self.description: CursorDescriptionType = []
 
-        # this is set to an iterator after a successfull query
-        self._results = None
+        # this is set to an iterator after a successful query
+        self._results: List[Tuple[Any, ...]] = []
 
-    @property
+    def custom_sql_to_method_dispatcher(self, command: str) -> Optional["BaseCursor"]:
+        """
+        Generic CUSTOM SQL dispatcher for internal methods
+        :param command: str
+        :return: None if no command found, or a Cursor with the result
+        """
+        method_name = self.custom_sql_to_method.get(command.lower())
+        return getattr(self, method_name)() if method_name else None
+
+    @property  # type: ignore
     @check_result
     @check_closed
-    def rowcount(self):
-        return len(self._results)
+    def rowcount(self) -> int:
+        """ Counts the number of rows on a result """
+        if self._results:
+            return len(self._results)
+        return 0
 
     @check_closed
-    def close(self):
+    def close(self) -> None:
         """Close the cursor."""
         self.closed = True
 
     @check_closed
-    def execute(self, operation, parameters=None):
+    def execute(self, operation, parameters=None) -> "BaseCursor":
+        """ Children must implement their own custom execute """
         raise NotImplementedError  # pragma: no cover
 
     @check_closed
@@ -143,7 +239,7 @@ class BaseCursor(object):
 
     @check_result
     @check_closed
-    def fetchone(self):
+    def fetchone(self) -> Optional[Tuple[Any, ...]]:
         """
         Fetch the next row of a query result set, returning a single sequence,
         or `None` when no more data is available.
@@ -155,7 +251,7 @@ class BaseCursor(object):
 
     @check_result
     @check_closed
-    def fetchmany(self, size=None):
+    def fetchmany(self, size: Optional[int] = None) -> List[Tuple[Any, ...]]:
         """
         Fetch the next set of rows of a query result, returning a sequence of
         sequences (e.g. a list of tuples). An empty sequence is returned when
@@ -167,7 +263,7 @@ class BaseCursor(object):
 
     @check_result
     @check_closed
-    def fetchall(self):
+    def fetchall(self) -> List[Tuple[Any, ...]]:
         """
         Fetch all (remaining) rows of a query result, returning them as a
         sequence of sequences (e.g. a list of tuples). Note that the cursor's
@@ -198,36 +294,37 @@ class BaseCursor(object):
 
     next = __next__
 
-    def sanitize_query(self, query):
-        # remove dummy schema from queries
+    def sanitize_query(self, query: str) -> str:
+        """
+        Removes dummy schema from queries
+        """
         return query.replace(f'FROM "{DEFAULT_SCHEMA}".', "FROM ")
 
-    def elastic_query(self, query: str, csv=False):
+    def elastic_query(self, query: str) -> Dict[str, Any]:
         """
         Request an http SQL query to elasticsearch
         """
-        self.description = None
         # Sanitize query
         query = self.sanitize_query(query)
         payload = {"query": query, "fetch_size": self.fetch_size}
-        if csv:
-            path = f"/{self.sql_path}/?format=csv"
-        else:
-            path = f"/{self.sql_path}/"
+        path = f"/{self.sql_path}/"
         try:
-            resp = self.es.transport.perform_request("POST", path, body=payload)
-        except es_exceptions.ConnectionError as e:
-            raise exceptions.OperationalError(
-                f"Error connecting to {self.url}: {e.info}"
-            )
-        except es_exceptions.RequestError as e:
+            response = self.es.transport.perform_request("POST", path, body=payload)
+        except es_exceptions.ConnectionError:
+            raise exceptions.OperationalError(f"Error connecting to Elasticsearch")
+        except es_exceptions.RequestError as ex:
             raise exceptions.ProgrammingError(
-                f"Error ({e.error}): {e.info['error']['reason']}"
+                f"Error ({ex.error}): {ex.info['error']['reason']}"
             )
-        return resp
+        # Opendistro errors are http status 200
+        if "error" in response:
+            raise exceptions.ProgrammingError(
+                f"({response['error']['reason']}): {response['error']['details']}"
+            )
+        return response
 
 
-def apply_parameters(operation, parameters):
+def apply_parameters(operation: str, parameters: Optional[Dict[str, Any]]) -> str:
     if parameters is None:
         return operation
 
